##standard curve function of x=log(concentration): y=f(x)=f(log(conc))
SDcurve=function(x,b1,b2,b3,b4) b2+(b1-b2)/(1+exp(b4*(x-log(b3))))
##

##inverse function of standard curve function: x=g(y=OD)
INV_SDcurve=function(y,b1,b2,b3,b4) b3*(((b1-b2)/(y-b2)-1)^(1/b4))
##


##################
######GLS for calibration curve, estimating regression parameters, variance parameter theta
Weighted_POM=function(obsY, obsX, parb1, parb2, parb3, parb4, parTheta)
{
     predY=parb2+(parb1-parb2)/(1+(obsX/parb3)^parb4)
     (obsY-predY)/(predY^parTheta)
}

Weighted_theta=function(obsY, obsX, parb1, parb2, parb3, parb4, parTheta)
{
     predY=parb2+(parb1-parb2)/(1+(obsX/parb3)^parb4)
     gmY=exp(mean(log(predY^parTheta)))
     (obsY-predY)/(predY^parTheta)*gmY
}


GLS_POM=function(data,b1_ini,b2_ini,b3_ini,theta_ini)
{
##intial estimate
fit_b=nls(od_cal ~ parb2+(parb1-parb2)/(1+(conc_cal_rep/parb3)^parb4),data,algorithm="port",start=list(parb1=b1_ini, parb2=b2_ini, parb3=b3_ini, parb4=1), lower=list(parb1=0.01,parb2=0.1,parb3=1,parb4=0.1), control=nls.control(warnOnly=TRUE))
estb1=coef(fit_b)[1]
estb2=coef(fit_b)[2]
estb3=coef(fit_b)[3]
estb4=coef(fit_b)[4]

data2=data.frame(data,estb1,estb2,estb3,estb4)
names(data2)=c("conc_cal_rep", "od_cal","parb1","parb2","parb3","parb4")
fit_theta=nls(~Weighted_theta(od_cal,conc_cal_rep,parb1,parb2,parb3,parb4,parTheta),data2,algorithm="port",start=list(parTheta=theta_ini), control=nls.control(warnOnly=TRUE))
estTheta=coef(fit_theta)[1]

##
iter=0
tolv=1
while(iter<=10 & tolv>=0.0001) {
theta_0=estTheta
data2=data.frame(data,theta_0 )
names(data2)=c("conc_cal_rep", "od_cal","parTheta")

fit_b=nls(~Weighted_POM(od_cal,conc_cal_rep,parb1,parb2,parb3,parb4,parTheta),data2,algorithm="port", start=list(parb1=estb1, parb2=estb2, parb3=estb3, parb4=estb4), lower=list(parb1=0.01,parb2=0.1,parb3=1,parb4=0.1), control=nls.control(warnOnly=TRUE))
estb1=coef(fit_b)[1]
estb2=coef(fit_b)[2]
estb3=coef(fit_b)[3]
estb4=coef(fit_b)[4]


data2=data.frame(data,estb1,estb2,estb3,estb4 )
names(data2)=c("conc_cal_rep", "od_cal","parb1","parb2","parb3","parb4")
fit_theta=nls(~Weighted_theta(od_cal,conc_cal_rep,parb1,parb2,parb3,parb4,parTheta),data2,algorithm="port",start=list(parTheta=theta_0), control=nls.control(warnOnly=TRUE))
estTheta=coef(fit_theta)[1]

iter=iter+1
tolv=abs(estTheta-theta_0)
}

rbind(estb1,estb2,estb3,estb4,estTheta)
}
##################


##################X matrix: gradient matrix based on standard concentrations
DQDB_Standard=function(X,parb1,parb2,parb3,parb4)
{
tmp1=1/(1+(X/parb3)^parb4)
tmp2=1-tmp1
tmp3=(parb1-parb2)*tmp1^2*((X/parb3)^parb4)*parb4/parb3
tmp4=-(parb1-parb2)*tmp1^2*((X/parb3)^parb4)*log(X/parb3)
cbind(tmp1,tmp2,tmp3,tmp4)
}


